import {render} from "preact";
import {useEffect, useState} from "preact/hooks";
import { Frame, Goroutine } from "./lib/goroutines";
import { useSignal, useComputed } from "@preact/signals";

const FrameView = (props: { frame: Frame }) => {
  const {frame} = props
  return <>
    <div class="flex flex-row gap-2">
      <div>
        {frame.Func}
      </div>
      <div>
        {frame.File}
      </div>
      <div>
        {frame.Line}
      </div>
    </div>
  </>

}

function Main() {
  //const [goroutines, setGoroutines] = useState<Goroutine[] | undefined>(undefined)
  const goroutines  = useSignal<Goroutine[]>([])


  const fetchGoroutines = async () => {
    const res = await fetch("./raw")
    const json = await res.json()
    let routines: Goroutine[] = json
    routines = routines.map(x=>{
      // divide by 1000000
      x.Wait = Number(BigInt(x.Wait) / BigInt(1000000))
      return x
    })
    goroutines.value = routines
  }

  useEffect(() => {
    fetchGoroutines()
  },[])

  useEffect(() => {
    if(!goroutines) {
      return
    }
    console.log(goroutines)
  },[goroutines])
  if(!goroutines) {
    return <>
      <div>
        fetching
      </div>
    </>
  }
  return <>
    <div class="flex flex-col">
      <div class="flex flex-row gap-2">
        <div>
          length:
        </div>
        <div>
          {goroutines.value.length}
        </div>
      </div>
      <div>
        {goroutines.value.map((x, idx)=>{


          return <div id={`id-${x.ID}`} class="flex flex-row gap-2">
            <div>
              {x.ID}
            </div>
            <div>
              {x.Wait}
            </div>
            <div>
              {x.Stack}
            </div>
            {x.CreatedBy &&
              <div>
                <FrameView frame={x.CreatedBy}/>
              </div>
            }
          </div>
        })}
      </div>
    </div>
  </>
}



const rootElement = document.getElementById("root")
if(rootElement) {
  render(<Main/>, rootElement);
}else {
  console.error("no root element found")
}
