export interface Frame {
    // Func is the name of the function, including package name, e.g. "main.main"
    // or "net/http.(*Server).Serve".
    Func: string;
    // File is the absolute path of source file e.g.
    // "/go/src/example.org/example/main.go".
    File: string;
    // Line is the line number of inside of the source file that was active when
    // the sample was taken.
    Line: number;
}

export interface Goroutine {
    // ID is the goroutine id (aka `goid`).
    ID: number;
    // State is the `atomicstatus` of the goroutine, or if "waiting" the
    // `waitreason`.
    State: string;
    // Wait is the approximate duration a goroutine has been waiting or in a
    // syscall as determined by the first gc after the wait started. Aka
    // `waitsince`.
    Wait: number;
    // LockedToThread is true if the goroutine is locked by a thread, aka
    // `lockedm`.
    LockedToThread: boolean;
    // Stack is the stack trace of the goroutine.
    Stack: Frame[];
    // FramesElided is true if the stack trace contains a message indicating that
    // additional frames were elided. This happens when the stack depth exceeds
    // 100.
    FramesElided: boolean;
    // CreatedBy is the frame that created this goroutine, nil for main().
    CreatedBy: Frame | null;
    // Ancestors are the Goroutines that created this goroutine.
    // See GODEBUG=tracebackancestors=n in https://pkg.go.dev/runtime.
    Ancestor?: Goroutine; // The `json:"Ancestor,omitempty"` tag is represented by the optional `?` in TypeScript
}
